unit Action;

interface

uses
  Classes, SysUtils, Graphics;

type
  TZonuActionList   = class;
  TZonuAction       = class;
  TZonuQuery        = class;
  TZonuCommandList  = class;
  TZonuCommand      = class;

  TZonuActionList = class(TList)
  private
    FBoardID: string;
  public
    //procedure Load(Board: TBoard); overload;
    //procedure Load; overload;
    //property  Board:  TBoard  read  FBoard;
    property  BoardID: string read  FBoardID  write FBoardID;
  end;

  TZonuCommandList = class(TList)
  private
  public
  end;

  //***********************************************
  //  ΏۃIuWFNg
  //***********************************************
  TZonuAction = class(TObject)
  private
    FQuery: TZonuQuery;
    FTargetBoards: TStringList;
    FTargetAllBoards: boolean;
    FCommandList: TZonuCommandList;
  public
    property  Query: TZonuQuery   read  FQuery  write FQuery;
    property  TargetBoards: TStringList read  FTargetBoards write FTargetBoards;
    property  TargetAllBoards: boolean  read  FTargetAllBoards  write FTargetAllBoards;
    property  Commands: TZonuCommandList  read  FCommandList  write FCommandList;
  end;

  TZonuActTitle = class(TZonuAction)
  private
  public
  end;

  TZonuActCount = class(TZonuAction)
  private
  public
  end;

  TZonuActHandle = class(TZonuAction)
  private
  public
  end;

  TZonuActMail = class(TZonuAction)
  private
  public
  end;

  TZonuActID = class(TZonuAction)
  private
  public
  end;

  TZonuActBody = class(TZonuAction)
  private
  public
  end;

  //***********************************************
  //  
  //***********************************************
  TZonuQuery  = class(TObject)
  private
  protected
    FDisplayText: string;
  public
    constructor Create; virtual;
    class function  GetDisplayText: string; virtual;
    property  DisplayText: string  read  FDisplayText;
  end;

  TZonuQryLike  = class(TZonuQuery)
  private
    FValue: string;
  public
    constructor Create; override;
    class function  GetDisplayText: string; override;
    property  Value: string   read  FValue  write FValue;
  end;

  TZonuQryEqual  = class(TZonuQuery)
  private
    FValue: string;
  public
    constructor Create; override;
    class function  GetDisplayText: string; override;
    property  Value: string   read  FValue  write FValue;
  end;

  TZonuQryLengthOver  = class(TZonuQuery)
  private
    FValue: integer;
  public
    constructor Create; override;
    property  Size: integer   read  FValue  write FValue;
  end;

  TZonuQryLengthUnder  = class(TZonuQuery)
  private
    FValue: integer;
  public
    constructor Create; override;
    property  Size: integer   read  FValue  write FValue;
  end;

  TZonuQryCountOver  = class(TZonuQuery)
  private
    FCount: integer;
  public
    constructor Create; override;
    property  Count: integer   read  FCount  write FCount;
  end;

  TZonuQryGikoBasic  = class(TZonuQuery)
  private
    FSourceFileName: string;
  public
    constructor Create; override;
    property  SourceFileName: string   read  FSourceFileName  write FSourceFileName;
  end;

  //***********************************************
  //  R}h
  //***********************************************
  TZonuCommand  = class(TObject)
  private
    FAction: TZonuAction;
  protected
    FDisplayText: string;
  public
    //procedure Execute;
    constructor Create; virtual;
    property  Action: TZonuAction read  FAction write FAction;
    property  DisplayText: string  read  FDisplayText;
  end;

  TZonuCmdHidden  = class(TZonuCommand)
  private
    FRelationHidden: boolean;
  public
    constructor Create; override;
    property  RelationHidden: boolean read  FRelationHidden write FRelationHidden;
  end;

  TZonuCmdPopup  = class(TZonuCommand)
  private
  public
    constructor Create; override;
  end;

  TZonuCmdMoveTo  = class(TZonuCommand)
  private
    FBoardID: string;
  public
    constructor Create; override;
    property  BoardID: string    read FBoardID  write FBoardID;
  end;

  TZonuCmdMoveToFav  = class(TZonuCmdMoveTo)
  private
  public
    constructor Create; override;
  end;

  TZonuCmdMoveToLocal  = class(TZonuCmdMoveTo)
  private
  public
    constructor Create; override;
  end;

  TZonuCmdAddBookmark  = class(TZonuCommand)
  private
  public
    constructor Create; override;
  end;

  TZonuCmdChangePriority  = class(TZonuCommand)
  private
    FValue: integer;
  public
    constructor Create; override;
    property  Value: integer  read  FValue  write FValue;
  end;

  TZonuCmdChangeMemo  = class(TZonuCommand)
  private
    FValue: string;
  public
    constructor Create; override;
    property  Value: string  read  FValue  write FValue;
  end;

  TZonuCmdChangeColor  = class(TZonuCommand)
  private
    FValue: TColor;
  public
    constructor Create; override;
    property  Value: TColor read  FValue  write FValue;
  end;

  TZonuCmdChangeValue  = class(TZonuCommand)
  private
    FValue: string;
  public
    constructor Create; override;
    property  Value: string read  FValue  write FValue;
  end;

  TZonuCmdGikoBasic  = class(TZonuCommand)
  private
    FSourceFileName: string;
  public
    constructor Create; override;
    property  SourceFileName: string read  FSourceFileName  write FSourceFileName;
  end;

implementation

{ TZonuQuery }

constructor TZonuQuery.Create;
begin
  FDisplayText  :=  GetDisplayText;
end;

class function TZonuQuery.GetDisplayText: string;
begin
  Result  :=  '';
end;

{ TZonuQryLike }

constructor TZonuQryLike.Create;
begin
  inherited;
end;

class function TZonuQryLike.GetDisplayText: string;
begin
  Result  :=  'w蕶܂ł';
end;

{ TZonuQryEqual }

constructor TZonuQryEqual.Create;
begin
  inherited;
end;

class function TZonuQryEqual.GetDisplayText: string;
begin
  FDisplayText  :=  'w蕶ƈv';
end;

{ TZonuQryLengthOver }

constructor TZonuQryLengthOver.Create;
begin
  inherited;
  FDisplayText  :=  'w蕶ȏゾ';
end;

{ TZonuQryLengthUnder }

constructor TZonuQryLengthUnder.Create;
begin
  inherited;
  FDisplayText  :=  'w蕶ȉ';
end;

{ TZonuQryCountOver }

constructor TZonuQryCountOver.Create;
begin
  inherited;
  FDisplayText  :=  'w背Xȏゾ';
end;

{ TZonuQryGikoBasic }

constructor TZonuQryGikoBasic.Create;
begin
  inherited;
  FDisplayText  :=  'MRBASICvOC';
end;

{ TZonuCommand }

constructor TZonuCommand.Create;
begin
  FDisplayText  :=  '';  
end;

{ TZonuCmdHidden }

constructor TZonuCmdHidden.Create;
begin
  inherited;
  FDisplayText  :=  '\ɂ';
end;

{ TZonuCmdPopup }

constructor TZonuCmdPopup.Create;
begin
  inherited;
  FDisplayText  :=  'm点';
end;

{ TZonuCmdMoveTo }

constructor TZonuCmdMoveTo.Create;
begin
  inherited;
  //
end;

{ TZonuCmdMoveToFav }

constructor TZonuCmdMoveToFav.Create;
begin
  inherited;
  FDisplayText  :=  'Cɓɓo^';
end;

{ TZonuCmdMoveToLocal }

constructor TZonuCmdMoveToLocal.Create;
begin
  inherited;
  FDisplayText  :=  '[JtH_Ɉړ';
end;

{ TZonuCmdAddBookmark }

constructor TZonuCmdAddBookmark.Create;
begin
  inherited;
  FDisplayText  :=  'o^';
end;

{ TZonuCmdChangePriority }

constructor TZonuCmdChangePriority.Create;
begin
  inherited;
  FDisplayText  :=  'Xbh̏dvxύX';
end;

{ TZonuCmdChangeMemo }

constructor TZonuCmdChangeMemo.Create;
begin
  inherited;
  FDisplayText  :=  'Xbh̃ύX';
end;

{ TZonuCmdChangeColor }

constructor TZonuCmdChangeColor.Create;
begin
  inherited;
  FDisplayText  :=  'FύX';
end;

{ TZonuCmdChangeValue }

constructor TZonuCmdChangeValue.Create;
begin
  inherited;
  FDisplayText  :=  'eύX';
end;

{ TZonuCmdGikoBasic }

constructor TZonuCmdGikoBasic.Create;
begin
  inherited;
  FDisplayText  :=  'MRBASICXNvgs';
end;

end.
