{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 NG Word Classes Unit                                              }
{                                                                             }
{ 2001-02 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Ngword;

interface

uses
  Classes, SysUtils, MonaUtils, Zonutils, ZonuClass;

type
  TNGWordMode = (nmHide, nmReplace);
  TCompareMode = (cmAgree , cmLike);
  TNGWordAttr = set of (ngID, ngHandle, ngBody, ngMail, ngThreadTitle);

  TNGWordList       = class;
  TNGWord           = class;
  TTargetBoardList  = class;
  TTargetBoard      = class;

  TNGWordList = class(TList)
  private
    FNGWordList : TNGWordList;
    function GetNGWord(index: integer): TNGWord;
    procedure SetNGWord(index: integer; const Value: TNGWord);
    procedure ParseNGWord(const Line: string; var NGWordData: TNGWord);
    procedure ParseNGTargetBoard(const Line: string; var NGTargetBoardData: TTargetBoard);
    function IsNG(Item: TNGWord; const s: string): boolean;
  public
    destructor Destroy; override;
    function  Add(Value: TNGWord): integer;
    function  IsNGHandle(Res: TRes; IsHint : boolean = False): boolean;
    function  IsNGID(Res: TRes; IsHint : boolean = False): boolean;
    function  IsNGMail(Res: TRes; IsHint: boolean = False): boolean;
    function  IsNGRes(Res: TRes; IsHint : boolean = False): boolean;
    function  ReplaceNGWord(Res: TRes): string;
    procedure Delete(Index: integer);
    procedure Clear; override;
    procedure LoadFromFile(const FileName: string);
    procedure SaveToFile(const FileName: string);
    //
    procedure BeginProc(const SiteID, BoardID: string);    //
    function  IsNGData(Res: TRes; var NGWord: TNGWord): boolean;
    procedure EndProc;
    //
    property  Items[index: integer]: TNGWord read GetNGWord write SetNGWord; default;
  end;

  TNGWord = class(TPersistent)
  private
    FText: string;
    FCompareMode: TCompareMode;
    FNGWordAttr: TNGWordAttr;
    FNGWordMode: TNGWordMode;
    FTargetBoardList: TTargetBoardList;
    FReplaceText: string;
    FHideInPopup: boolean;
    FRelationAbone: boolean;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property  NGWordAttr: TNGWordAttr read  FNGWordAttr write FNGWordAttr;
    property  NGWordMode: TNGWordMode read  FNGWordMode write FNGWordMode;
    property  Text: string  read  FText write FText;
    property  CompareMode: TCompareMode read  FCompareMode  write FCompareMode;
    property  ReplaceText: string read  FReplaceText  write FReplaceText;
    property  TargetBoardList: TTargetBoardList read  FTargetBoardList  write FTargetBoardList;
    property  HideInPopup:  boolean read  FHideInPopup  write FHideInPopup;
    property  RelationAbone:  boolean read  FRelationAbone  write FRelationAbone;
  end;

  TTargetBoardList  = class(TList)
  private
    function GetTargetData(index: integer): TTargetBoard;
    procedure SetTargetData(index: integer; const Value: TTargetBoard);
  public
    destructor Destroy; override;
    function  Add(Value: TTargetBoard): integer;
    procedure Delete(Index: integer);
    procedure Clear; override;
    property  Items[index: integer]: TTargetBoard read GetTargetData write SetTargetData; default;
  end;

  TTargetBoard  = class(TPersistent)
  private
    FIsAll: boolean;
    FSiteID: string;
    FBoardID: string;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property  SiteID: string   read  FSiteID  write FSiteID;
    property  BoardID: string   read  FBoardID  write FBoardID;
    property  IsAll: boolean  read  FIsAll  write FIsAll;
  end;


implementation

////////////////////////////////////////////////////////////////////////////////
{ TNGWordList }
////////////////////////////////////////////////////////////////////////////////

function TNGWordList.Add(Value: TNGWord): integer;
begin
  Result  :=  inherited Add(Value);
end;

procedure TNGWordList.Clear;
var
  i : integer;
begin
  for i :=  Count - 1 downto  0 do  begin
    Delete(i);
  end;
  inherited;
end;

procedure TNGWordList.Delete(Index: integer);
begin
  TNGWord(Items[Index]).Free;
  inherited Delete(Index);
end;

destructor TNGWordList.Destroy;
begin
  Clear;
  inherited;
end;

function TNGWordList.GetNGWord(index: integer): TNGWord;
begin
  Result  :=  TNGWord(inherited Items[index]);
end;

procedure TNGWordList.ParseNGTargetBoard(const Line: string;
  var NGTargetBoardData: TTargetBoard);
begin
  NGTargetBoardData.SiteID    :=  ChooseString(Line, #9, 1);
  NGTargetBoardData.BoardID   :=  ChooseString(Line, #9, 2);
  if  (NGTargetBoardData.SiteID <> '#ALL')  then  begin
    NGTargetBoardData.IsAll   :=  False;
  end else begin
    NGTargetBoardData.IsAll   :=  True;
  end;
end;

procedure TNGWordList.ParseNGWord(const Line: string;
  var NGWordData: TNGWord);
var
  s : string;
begin
  NGWordData.Text           :=  ChooseString(Line, #9, 0);
  if  (ChooseString(Line, #9, 1) = '1') then  begin
    NGWordData.NGWordAttr   :=  NGWordData.NGWordAttr + [ngHandle];
  end;
  if  (ChooseString(Line, #9, 2) = '1') then  begin
    NGWordData.NGWordAttr   :=  NGWordData.NGWordAttr + [ngID];
  end;
  if  (ChooseString(Line, #9, 3) = '1') then  begin
    NGWordData.NGWordAttr   :=  NGWordData.NGWordAttr + [ngBody];
  end;
  NGWordData.NGWordMode     :=  TNGWordMode(StrToIntDef(ChooseString(Line, #9, 4), 0));
  NGWordData.ReplaceText    :=  ChooseString(Line, #9, 5);
  s :=  ChooseString(Line, #9, 6);
  if  (s = '')  then  begin
    NGWordData.CompareMode  :=  cmLike;
  end else begin
    NGWordData.CompareMode  :=  TCompareMode(StrToIntDef(s, 1));
  end;
  if  (ChooseString(Line, #9, 7) = '1') then  begin
    NGWordData.NGWordAttr   :=  NGWordData.NGWordAttr + [ngMail];
  end;
  if  (ChooseString(Line, #9, 10) = '1') then  begin
    NGWordData.NGWordAttr   :=  NGWordData.NGWordAttr + [ngThreadTitle];
  end;
  NGWordData.HideInPopup    :=  (ChooseString(Line, #9, 8) = '1');
  NGWordData.RelationAbone  :=  (ChooseString(Line, #9, 9) = '1');
end;

procedure TNGWordList.SetNGWord(index: integer; const Value: TNGWord);
begin
  inherited Items[index]  :=  Value;
end;

procedure TNGWordList.LoadFromFile(const FileName: string);
var
  List : TStringList;
  ANGWord : TNGWord;
  ATargetBoard : TTargetBoard;
  i : integer;
begin
  Clear;
  if  (FileExists(FileName))  then  begin
    List  :=  TStringList.Create;
    List.LoadFromFile(FileName);
    //obNAbv
    if  (List.Count > 0)  then  begin
      List.SaveToFile(ChangeFileExt(FileName, '.bak')); 
    end;
    ANGWord :=  nil;
    for i :=  0 to  List.Count - 1  do  begin
      if  (List[i] <> '') then  begin
        if  (List[i][1] <> #9) then  begin
          ANGWord :=  TNGWord.Create;
          ParseNGWord(List[i], ANGWord);
          Add(ANGWord);
        end else begin
          if  (ANGWord <> nil)  then  begin
            ATargetBoard  :=  TTargetBoard.Create;
            ParseNGTargetBoard(List[i], ATargetBoard);
            ANGWord.TargetBoardList.Add(ATargetBoard);
          end;
        end;
      end;
    end;
    List.Free;
  end;
end;

procedure TNGWordList.SaveToFile(const FileName: string);
  function BoolToStr(const b: boolean): string;
  begin
    if  (b) then  begin
      Result  :=  '1';
    end else begin
      Result  :=  '0';
    end;
  end;
var
  List : TStringList;
  i, x : integer;
begin
  List  :=  TStringList.Create;
  for i :=  0 to  Count - 1 do  begin
    List.Add(Items[i].Text + #9  +
             BoolToStr(ngHandle in Items[i].NGWordAttr)   + #9 +
             BoolToStr(ngID     in Items[i].NGWordAttr)   + #9 +
             BoolToStr(ngBody   in Items[i].NGWordAttr)   + #9 +
             IntToStr(Integer(Items[i].NGWordMode))       + #9 +
             Items[i].ReplaceText + #9 +
             IntToStr(Integer(Items[i].CompareMode)) + #9 +
             BoolToStr(ngMail in Items[i].NGWordAttr) + #9 +
             BoolToStr(Items[i].HideInPopup) + #9 +
             BoolToStr(Items[i].RelationAbone) + #9 +
             BoolToStr(ngThreadTitle in Items[i].NGWordAttr)
             );
    for x :=  0 to  Items[i].TargetBoardList.Count - 1  do  begin
      if  (Items[i].TargetBoardList[x].IsAll) then  begin
        List.Add(#9 + '#ALL');
      end else begin
        List.Add(#9 +
                 Items[i].TargetBoardList[x].SiteID +
                 #9 +
                 Items[i].TargetBoardList[x].BoardID);
      end;
    end;
  end;
  List.SaveToFile(FileName);
  List.Free;
end;

function TNGWordList.IsNG(Item: TNGWord; const s : string): boolean;
begin
  Result  :=  False;
  if  (Item.CompareMode = cmLike) then  begin
    if  (AnsiPos(Item.Text, TrimTag(s)) > 0)  then  begin
      Result  :=  True;
    end;
  end else begin
    if  (Trim(Item.Text) = TrimTag(s))  then  begin
      Result  :=  True;
    end;
  end;
end;


function TNGWordList.IsNGHandle(Res: TRes; IsHint : boolean = False): boolean;
var
  x : integer;
begin
  Result  :=  False;
  for x :=  0 to  Count - 1  do  begin
    if  (ngHandle in Items[x].NGWordAttr) then  begin
      if  (IsNG(Items[x], TrimTag(Res.Name)))  then  begin
        if  (IsHint) and (not Items[x].HideInPopup) then  begin
          Exit;
        end;
        Result  :=  True;
        Break;
      end;
    end;
  end;
end;

function TNGWordList.IsNGID(Res: TRes; IsHint : boolean = False): boolean;
var
  x : integer;
begin
  Result  :=  False;
  for x :=  0 to  Count - 1  do  begin
    if  (ngID in Items[x].NGWordAttr) then  begin
      if  (IsNG(Items[x], Res.ID))  then  begin
        if  (IsHint) and (not Items[x].HideInPopup) then  begin
          Exit;
        end;
        Result  :=  True;
        Break;
      end;
    end;
  end;
end;

function TNGWordList.IsNGMail(Res: TRes; IsHint : boolean = False): boolean;
var
  x : integer;
begin
  Result  :=  False;
  for x :=  0 to  Count - 1  do  begin
    if  (ngMail in Items[x].NGWordAttr) then  begin
      if  (IsNG(Items[x], Res.Mail))  then  begin
        if  (IsHint) and (not Items[x].HideInPopup) then  begin
          Exit;
        end;
        Result  :=  True;
        Break;
      end;
    end;
  end;
end;

function TNGWordList.IsNGRes(Res: TRes; IsHint : boolean = False): boolean;
var
  x : integer;
begin
  Result  :=  False;
  for x :=  0 to  Count - 1  do  begin
    if  (ngBody in Items[x].NGWordAttr) then  begin
      if  (Items[x].NGWordMode = nmHide) then begin
        if  (IsNG(Items[x], Res.Body))  then  begin
          if  (IsHint) and (not Items[x].HideInPopup) then  begin
            Exit;
          end;
          Result  :=  True;
          Break;
        end;
      end;
    end;
  end;
end;

function TNGWordList.ReplaceNGWord(Res: TRes): string;
var
  x : integer;
  s : string;
begin
  s := Res.Body;
  for x :=  0 to  Count - 1  do  begin
    if  (ngBody in Items[x].NGWordAttr) then  begin
      if  (Items[x].NGWordMode = nmReplace) then begin
        s := StringReplace(Res.Body,
                           Items[x].Text,
                           Items[x].ReplaceText,
                           [rfReplaceAll, rfIgnoreCase]);
      end;
    end;
  end;
  Result  :=  s;
end;

////////////////////////////////////////////////////////////////////////////////
{ TTargetBoardList }
////////////////////////////////////////////////////////////////////////////////

function TTargetBoardList.Add(Value: TTargetBoard): integer;
begin
  Result  :=  inherited Add(Value);
end;

procedure TTargetBoardList.Clear;
var
  i : integer;
begin
  for i :=  Count - 1 downto  0 do  begin
    Delete(i);
  end;
  inherited;
end;

procedure TTargetBoardList.Delete(Index: integer);
begin
  TTargetBoard(Items[Index]).Free;
  inherited Delete(Index);
end;

destructor TTargetBoardList.Destroy;
begin
  Clear;
  inherited;
end;

function TTargetBoardList.GetTargetData(index: integer): TTargetBoard;
begin
  Result  :=  TTargetBoard(inherited Items[index]);
end;

procedure TTargetBoardList.SetTargetData(index: integer;
  const Value: TTargetBoard);
begin
  inherited Items[index]  :=  Value;
end;

////////////////////////////////////////////////////////////////////////////////
{ TNGWord }
////////////////////////////////////////////////////////////////////////////////

constructor TNGWord.Create;
begin
  FNGWordAttr   :=  [];
  FNGWordMode   :=  nmHide;
  FText         :=  '';
  FCompareMode  :=  cmAgree;
  FTargetBoardList  :=  TTargetBoardList.Create;
  FHideInPopup  :=  False;
  FRelationAbone:=  False;
end;

destructor TNGWord.Destroy;
begin
  FTargetBoardList.Clear;
  FTargetBoardList.Free;
  inherited;
end;

procedure TNGWord.Assign(Source: TPersistent);
var
  ATargetBoard : TTargetBoard;
  i : integer;
begin
  if  (Source is TNGWord) then  begin
    FNGWordAttr   :=  TNGWord(Source).NGWordAttr;
    FNGWordMode   :=  TNGWord(Source).NGWordMode;
    FText         :=  TNGWord(Source).Text;
    FCompareMode  :=  TNGWord(Source).CompareMode;
    FRelationAbone:=  TNGWord(Source).RelationAbone;
    FTargetBoardList.Clear;
    for i :=  0 to  TNGWord(Source).TargetBoardList.Count - 1 do  begin
      ATargetBoard  :=  TTargetBoard.Create;
      ATargetBoard.Assign(TNGWord(Source).TargetBoardList[i]);
      FTargetBoardList.Add(ATargetBoard);
    end;
    FHideInPopup  :=  TNGWord(Source).HideInPopup;
  end else begin
    inherited Assign(Source);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
{ TTargetBoard }
////////////////////////////////////////////////////////////////////////////////

constructor TTargetBoard.Create;
begin
  FSiteID     :=  '';
  FBoardID    :=  '';
  FIsAll      :=  False;
end;

procedure TTargetBoard.Assign(Source: TPersistent);
begin
  if  (Source is TTargetBoard)  then  begin
    FSiteID     :=  TTargetBoard(Source).SiteID;
    FBoardID    :=  TTargetBoard(Source).BoardID;
    FIsAll      :=  TTargetBoard(Source).IsAll;
  end else begin
    inherited Assign(Source);
  end;
end;

procedure TNGWordList.BeginProc(const SiteID, BoardID: string);
var
  ANGWord : TNGWord;
  i, x : integer;
begin
  FNGWordList :=  TNGWordList.Create;
  for i :=  0 to  Count - 1 do  begin
    for x :=  0 to  Items[i].TargetBoardList.Count - 1  do  begin
      if  ((Items[i].TargetBoardList[x].SiteID  = SiteID) and
           (Items[i].TargetBoardList[x].BoardID = BoardID)) or
          (Items[i].TargetBoardList[x].IsAll) then
      begin
        ANGWord :=  TNGWord.Create;
        ANGWord.Assign(Items[i]);
        FNGWordList.Add(ANGWord);
        Break;
      end;
    end;
  end;
end;

procedure TNGWordList.EndProc;
begin
  FNGWordList.Clear;
end;

function TNGWordList.IsNGData(Res: TRes; var NGWord: TNGWord): boolean;
var
  x : integer;
begin
  Result  :=  False;
  for x :=  0 to  FNGWordList.Count - 1  do  begin
    if  (ngHandle in FNGWordList[x].NGWordAttr) then  begin
      if  (IsNG(FNGWordList[x], TrimTag(Res.Name)))  then  begin
        NGWord  :=  FNGWordList[x];
        Result  :=  True;
        Exit;
      end;
    end;
    if  (ngID in FNGWordList[x].NGWordAttr) then  begin
      if  (IsNG(FNGWordList[x], Res.ID))  then  begin
        NGWord  :=  FNGWordList[x];
        Result  :=  True;
        Exit;
      end;
    end;
    if  (ngMail in FNGWordList[x].NGWordAttr) then  begin
      if  (IsNG(FNGWordList[x], Res.Mail))  then  begin
        NGWord  :=  FNGWordList[x];
        Result  :=  True;
        Exit;
      end;
    end;
    if  (ngBody in FNGWordList[x].NGWordAttr) then  begin
      if  (FNGWordList[x].NGWordMode = nmHide) then begin
        if  (IsNG(FNGWordList[x], Res.Body))  then  begin
          NGWord  :=  FNGWordList[x];
          Result  :=  True;
          Exit;
        end;
      end;
    end;
  end;
end;

end.
