{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 Action Classes Unit                                               }
{                                                                             }
{ 2001-02 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuAction;

interface

uses
  Classes, SysUtils, MonaUtils, Zonutils, ZonuClass, Hzbconst;

type
  TActionTarget = set of  (atResHandle,
                           atResMail,
                           atResID,
                           atResBody,
                           atThreadTitle,
                           atThreadResCount,
                           atThreadLoadDate
                          );
  TActionType = (atAbone,
                 atReplace,
                 atPopup,
                 atAddToFav,
                 atMoveToLocal,
                 atBookmark,
                 atSetColor,
                 atSetReverse,
                 atSetBold,
                 atSetKakoLog,
                 atSetPriority
                );

  TActionWhere  = (awLike,
                   awSame,
                   awLengthLT,
                   awLengthGT,
                   awCountGT,
                   awDateGt,
                   awNonGetCountGT);

  TZonuActionList   = class;
  TZonuAction       = class;
  TTargetBoardList  = class;
  TTargetBoard      = class;

  TZonuActionList = class(TList)
  private
    //FActionList: TZonuActionList;
    function GetAction(index: integer): TZonuAction;
    procedure SetAction(index: integer; const Value: TZonuAction);
    procedure ParseNGWord(const Line: string; var ac: TZonuAction);
    procedure ParseNGTargetBoard(const Line: string; var tb: TTargetBoard);
  public
    constructor Create;
    destructor Destroy; override;
    function  Add(Value: TZonuAction): integer;
    procedure Delete(Index: integer);
    procedure Clear; override;
    procedure LoadFromFile(const FileName: string);
    procedure SaveToFile(const FileName: string);

    //procedure BeginProc(const SiteID, BoardID: string);
    function  CheckRes(Res: TRes; SiteID, BoardID: string;
      var Action: TZonuAction; var CheckBegin: integer): boolean;
    function  CheckThread(ThreadItem: TThreadItem; SiteID, BoardID: string;
      var Action: TZonuAction): boolean;
    //procedure EndProc;

    property  Items[index: integer]: TZonuAction  read GetAction write SetAction; default;
  end;

  TZonuAction = class(TPersistent)
  private
    FTargetBoardList: TTargetBoardList;
    FTarget: TActionTarget;
    FActionType: TActionType;
    FActionWhere: TActionWhere;
    FParam1: string;
    FParam2: string;
    FAboneHideInPopup: boolean;
    FAboneRelation: boolean;
    FNewThreadOnly: boolean;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function  GetTargetText: string;
    function  GetActionTypeText: string;
    function  GetActionWhereText: string;
    property  Target: TActionTarget read  FTarget write FTarget;
    property  ActionType: TActionType read  FActionType write FActionType;
    property  ActionWhere:  TActionWhere  read  FActionWhere  write FActionWhere;
    property  Param1: string  read  FParam1 write FParam1;
    property  Param2: string  read  FParam2 write FParam2;
    property  AboneHideInPopup: boolean read  FAboneHideInPopup write FAboneHideInPopup;
    property  AboneRelation: boolean  read  FAboneRelation  write FAboneRelation;
    property  TargetBoardList: TTargetBoardList read  FTargetBoardList  write FTargetBoardList;
    property  NewThreadOnly: boolean  read  FNewThreadOnly  write FNewThreadOnly;    
  end;

  TTargetBoardList  = class(TList)
  private
    function GetTargetData(index: integer): TTargetBoard;
    procedure SetTargetData(index: integer; const Value: TTargetBoard);
  public
    destructor Destroy; override;
    function  Add(Value: TTargetBoard): integer;
    procedure Delete(Index: integer);
    procedure Clear; override;
    property  Items[index: integer]: TTargetBoard read GetTargetData write SetTargetData; default;
  end;

  TTargetBoard  = class(TPersistent)
  private
    FIsAll: boolean;
    FSiteID: string;
    FBoardID: string;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property  SiteID: string   read  FSiteID  write FSiteID;
    property  BoardID: string   read  FBoardID  write FBoardID;
    property  IsAll: boolean  read  FIsAll  write FIsAll;
  end;

implementation

////////////////////////////////////////////////////////////////////////////////
{ TTargetBoardList }
////////////////////////////////////////////////////////////////////////////////

function TTargetBoardList.Add(Value: TTargetBoard): integer;
begin
  Result  :=  inherited Add(Value);
end;

procedure TTargetBoardList.Clear;
var
  i : integer;
begin
  for i :=  Count - 1 downto  0 do  begin
    Delete(i);
  end;
  inherited;
end;

procedure TTargetBoardList.Delete(Index: integer);
begin
  TTargetBoard(Items[Index]).Free;
  inherited Delete(Index);
end;

destructor TTargetBoardList.Destroy;
begin
  Clear;
  inherited;
end;

function TTargetBoardList.GetTargetData(index: integer): TTargetBoard;
begin
  Result  :=  TTargetBoard(inherited Items[index]);
end;

procedure TTargetBoardList.SetTargetData(index: integer;
  const Value: TTargetBoard);
begin
  inherited Items[index]  :=  Value;
end;

////////////////////////////////////////////////////////////////////////////////
{ TTargetBoard }
////////////////////////////////////////////////////////////////////////////////

constructor TTargetBoard.Create;
begin
  FSiteID     :=  '';
  FBoardID    :=  '';
  FIsAll      :=  False;
end;

procedure TTargetBoard.Assign(Source: TPersistent);
begin
  if  (Source is TTargetBoard)  then  begin
    FSiteID     :=  TTargetBoard(Source).SiteID;
    FBoardID    :=  TTargetBoard(Source).BoardID;
    FIsAll      :=  TTargetBoard(Source).IsAll;
  end else begin
    inherited Assign(Source);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
{ TZonuActionList }
////////////////////////////////////////////////////////////////////////////////

constructor TZonuActionList.Create;
begin

end;

destructor TZonuActionList.Destroy;
begin
  Clear;
  inherited;
end;

function TZonuActionList.Add(Value: TZonuAction): integer;
begin
  Result  :=  inherited Add(Value);
end;

procedure TZonuActionList.Clear;
var
  i : integer;
begin
  for i :=  Count - 1 downto  0 do  begin
    Delete(i);
  end;
  inherited;
end;

procedure TZonuActionList.Delete(Index: integer);
begin
  TZonuAction(Items[Index]).Free;
  inherited Delete(Index);
end;

function TZonuActionList.GetAction(index: integer): TZonuAction;
begin
  Result  :=  TZonuAction(inherited Items[index]);
end;

procedure TZonuActionList.SetAction(index: integer;
  const Value: TZonuAction);
begin
  inherited Items[index]  :=  Value;
end;

procedure TZonuActionList.LoadFromFile(const FileName: string);
var
  sl  : TStringList;
  ac  : TZonuAction;
  tb  : TTargetBoard;
  i : integer;
begin
  Clear;
  if  (FileExists(FileName))  then  begin
    sl    :=  TStringList.Create;
    sl.LoadFromFile(FileName);
    if  (sl.Count > 0)  then  begin
      //obNAbv
      sl.SaveToFile(ChangeFileExt(FileName, '.bak'));
    end;
    ac  :=  nil;
    for i :=  0 to  sl.Count - 1  do  begin
      if  (sl[i] <> '') then  begin
        if  (sl[i][1] <> #9) then  begin
          //ANV
          ac  :=  TZonuAction.Create;
          ParseNGWord(sl[i], ac);
          Add(ac);
        end else begin
          //Ώ۔
          if  (ac <> nil) then  begin
            tb  :=  TTargetBoard.Create;
            ParseNGTargetBoard(sl[i], tb);
            ac.TargetBoardList.Add(tb);
          end;
        end;
      end;
    end;
    sl.Free;
  end;
end;

procedure TZonuActionList.SaveToFile(const FileName: string);
  function BoolToStr(const b: boolean): string;
  begin
    if  (b) then  begin
      Result  :=  '1';
    end else begin
      Result  :=  '0';
    end;
  end;
var
  List : TStringList;
  i, x : integer;
begin
  List  :=  TStringList.Create;
  for i :=  0 to  Count - 1 do  begin
    List.Add(
             Items[i].GetTargetText + #9 +
             Items[i].GetActionTypeText + #9  +
             Items[i].GetActionWhereText  + #9  +
             Items[i].Param1  + #9  +
             Items[i].Param2  + #9  +
             BoolToStr(Items[i].AboneHideInPopup) + #9  +
             BoolToStr(Items[i].AboneRelation) + #9 +
             BoolToStr(Items[i].NewThreadOnly)
             );
    for x :=  0 to  Items[i].TargetBoardList.Count - 1  do  begin
      if  (Items[i].TargetBoardList[x].IsAll) then  begin
        List.Add(#9 + '#ALL');
      end else begin
        List.Add(#9 +
                 Items[i].TargetBoardList[x].SiteID +
                 #9 +
                 Items[i].TargetBoardList[x].BoardID);
      end;
    end;
  end;
  List.SaveToFile(FileName);
  List.Free;
end;

procedure TZonuActionList.ParseNGTargetBoard(const Line: string;
  var tb: TTargetBoard);
begin
  tb.SiteID    :=  ChooseString(Line, #9, 1);
  tb.BoardID   :=  ChooseString(Line, #9, 2);
  tb.IsAll     :=  (tb.SiteID = '#ALL');
end;

procedure TZonuActionList.ParseNGWord(const Line: string;
  var ac: TZonuAction);
var
  s : string;
  sl: TStringList;
  i : integer;
begin
  //^[Qbg
  ac.Target :=  [];
  sl        :=  TStringList.Create;
  sl.CommaText   :=  ChooseString(Line, #9, 0);
  for i :=  0 to  sl.Count - 1  do  begin
    if  (SameText(sl[i], 'RESHANDLE'))  then  begin
      ac.Target :=  ac.Target + [atResHandle];
    end else
    if  (SameText(sl[i], 'RESMAIL'))  then  begin
      ac.Target :=  ac.Target + [atResMail];
    end else
    if  (SameText(sl[i], 'RESID'))  then  begin
      ac.Target :=  ac.Target + [atResID];
    end else
    if  (SameText(sl[i], 'RESBODY'))  then  begin
      ac.Target :=  ac.Target + [atResBody];
    end else
    if  (SameText(sl[i], 'THREADTITLE'))  then  begin
      ac.Target :=  ac.Target + [atThreadTitle];
    end else
    if  (SameText(sl[i], 'THREADRESCOUNT'))  then  begin
      ac.Target :=  ac.Target + [atThreadResCount];
    end else
    if  (SameText(sl[i], 'THREADLOADDATE'))  then  begin
      ac.Target :=  ac.Target + [atThreadLoadDate];
    end;
  end;
  sl.Free;
  //ANV̎
  s :=  AnsiUpperCase(ChooseString(Line, #9, 1));
  if  (s  = 'ABONE')  then  begin
    ac.ActionType  :=  atAbone;
  end else
  if  (s  = 'REPLACE')  then  begin
    ac.ActionType  :=  atReplace;
  end else
  if  (s  = 'POPUP')  then  begin
    ac.ActionType  :=  atPopup;
  end else
  if  (s  = 'ADDTOFAV')  then  begin
    ac.ActionType  :=  atAddToFav;
  end else
  if  (s  = 'MOVETOLOCAL')  then  begin
    ac.ActionType  :=  atMoveToLocal;
  end else
  if  (s  = 'BOOKMARK')  then  begin
    ac.ActionType  :=  atBookmark;
  end else
  if  (s  = 'SETCOLOR')  then  begin
    ac.ActionType  :=  atSetColor;
  end else
  if  (s  = 'SETREVERSE')  then  begin
    ac.ActionType  :=  atSetReverse;
  end else
  if  (s  = 'SETBOLD')  then  begin
    ac.ActionType  :=  atSetBold;
  end else
  if  (s  = 'SETKAKOLOG')  then  begin
    ac.ActionType  :=  atSetKakoLog;
  end;
  //
  s :=  AnsiUpperCase(ChooseString(Line, #9, 2));
  if  (s  = 'LIKE')  then  begin
    ac.ActionWhere  :=  awLike;
  end else
  if  (s  = 'SAME')  then  begin
    ac.ActionWhere  :=  awSame;
  end else
  if  (s  = 'LENGTHLT')  then  begin
    ac.ActionWhere  :=  awLengthLT;
  end else
  if  (s  = 'LENGTHGT')  then  begin
    ac.ActionWhere  :=  awLengthGT;
  end else
  if  (s  = 'COUNTGT')  then  begin
    ac.ActionWhere  :=  awCountGT;
  end else
  if  (s  = 'DATEGT')  then  begin
    ac.ActionWhere  :=  awDateGT;
  end else
  if  (s  = 'NONGETCOUNTGT')  then  begin
    ac.ActionWhere  :=  awNonGetCountGT;
  end;
  //p[^1
  ac.Param1 :=  ChooseString(Line, #9, 3);
  ac.Param2 :=  ChooseString(Line, #9, 4);

  //ځ[̏ꍇ
  if  (ac.ActionType  = atAbone)  then  begin
    ac.AboneHideInPopup   :=  (ChooseString(Line, #9, 5) = '1');
    ac.AboneRelation      :=  (ChooseString(Line, #9, 6) = '1');
  end;
  ac.NewThreadOnly        :=  (ChooseString(Line, #9, 7) = '1');
end;

////////////////////////////////////////////////////////////////////////////////
{ TZonuAction }
////////////////////////////////////////////////////////////////////////////////

procedure TZonuAction.Assign(Source: TPersistent);
var
  ATargetBoard : TTargetBoard;
  i : integer;
begin
  if  (Source is TZonuAction) then  begin
    FTarget       :=  TZonuAction(Source).Target;
    FActionType   :=  TZonuAction(Source).ActionType;
    FActionWhere  :=  TZonuAction(Source).ActionWhere;
    FParam1       :=  TZonuAction(Source).Param1;
    FParam2       :=  TZonuAction(Source).Param2;
    FAboneHideInPopup   :=  TZonuAction(Source).AboneHideInPopup;
    FAboneRelation      :=  TZonuAction(Source).AboneRelation;
    FTargetBoardList.Clear;
    for i :=  0 to  TZonuAction(Source).TargetBoardList.Count - 1 do  begin
      ATargetBoard  :=  TTargetBoard.Create;
      ATargetBoard.Assign(TZonuAction(Source).TargetBoardList[i]);
      FTargetBoardList.Add(ATargetBoard);
    end;
  end else begin
    inherited Assign(Source);
  end;
end;

constructor TZonuAction.Create;
begin
  FTarget       :=  [];
  FActionType   :=  atAbone;
  FActionWhere  :=  awLike;
  FParam1       :=  '';
  FParam2       :=  '';
  FAboneHideInPopup :=  False;
  FAboneRelation    :=  False;
  FTargetBoardList  :=  TTargetBoardList.Create;
end;

destructor TZonuAction.Destroy;
begin
  FTargetBoardList.Clear;
  FTargetBoardList.Free;
  inherited;
end;

function TZonuAction.GetActionTypeText: string;
begin
  if  (ActionType = atAbone) then  begin
    Result  :=  'ABONE';
  end else
  if  (ActionType = atReplace) then  begin
    Result  :=  'REPLACE';
  end else
  if  (ActionType = atPopup) then  begin
    Result  :=  'POPUP';
  end else
  if  (ActionType = atAddToFav) then  begin
    Result  :=  'ADDTOFAV';
  end else
  if  (ActionType = atMoveToLocal) then  begin
    Result  :=  'MOVETOLOCAL';
  end else
  if  (ActionType = atBookmark) then  begin
    Result  :=  'BOOKMARK';
  end else
  if  (ActionType = atSetColor) then  begin
    Result  :=  'SETCOLOR';
  end else
  if  (ActionType = atSetReverse) then  begin
    Result  :=  'SETREVERSE';
  end else
  if  (ActionType = atSetBold) then  begin
    Result  :=  'SETBOLD';
  end else
  if  (ActionType = atSetKakoLog) then  begin
    Result  :=  'SETKAKOLOG';
  end;
end;

function TZonuAction.GetActionWhereText: string;
begin
  if  (ActionWhere = awLike) then  begin
    Result  :=  'LIKE';
  end else
  if  (ActionWhere = awSame) then  begin
    Result  :=  'SAME';
  end else
  if  (ActionWhere = awLengthLT) then  begin
    Result  :=  'LENGTHLT';
  end else
  if  (ActionWhere = awLengthGT) then  begin
    Result  :=  'LENGTHGT';
  end else
  if  (ActionWhere = awCountGT) then  begin
    Result  :=  'COUNTGT';
  end else
  if  (ActionWhere = awDateGT) then  begin
    Result  :=  'DATEGT';
  end else
  if  (ActionWhere = awNonGetCountGT) then  begin
    Result  :=  'NONGETCOUNTGT';
  end;
end;

function TZonuAction.GetTargetText: string;
var
  s: string;
begin
  s :=  '';
  if  (atResHandle in Target) then  begin
    s :=  s + ',RESHANDLE';
  end;
  if  (atResMail in Target) then  begin
    s :=  s + ',RESMAIL';
  end;
  if  (atResID in Target) then  begin
    s :=  s + ',RESID';
  end;
  if  (atResBody in Target) then  begin
    s :=  s + ',RESBODY';
  end;
  if  (atThreadTitle in Target) then  begin
    s :=  s + ',THREADTITLE';
  end;
  if  (atThreadResCount in Target) then  begin
    s :=  s + ',THREADRESCOUNT';
  end;
  if  (atThreadLoadDate in Target) then  begin
    s :=  s + ',THREADLOADDATE';
  end;
  if  (s <> '') then  s :=  Copy(s, 2, Length(s));
  Result  :=  s;
end;

{
procedure TZonuActionList.BeginProc(const SiteID, BoardID: string);
var
  za: TZonuAction;
  i, x : integer;
begin
  for i :=  0 to  Count - 1 do  begin
    for x :=  0 to  Items[i].TargetBoardList.Count - 1  do  begin
      if  ((Items[i].TargetBoardList[x].SiteID  = SiteID) and
           (Items[i].TargetBoardList[x].BoardID = BoardID)) or
          (Items[i].TargetBoardList[x].IsAll) then
      begin
        za  :=  TZonuAction.Create;
        za.Assign(Items[i]);
        ActionList.Add(za);
        Break;
      end;
    end;
  end;
end;
}
function TZonuActionList.CheckRes(Res: TRes; SiteID, BoardID: string;
  var Action: TZonuAction; var CheckBegin: integer): boolean;

  function  _IsNG(a: TZonuAction; const s: string): boolean;
  var
    s1: string;
    s2: string;
  begin
    s1  :=  DecodeHtmlEsc(s);
    s2  :=  StringReplace(a.Param1, '#13#10', #13#10, [rfReplaceAll]);
    if  (a.ActionWhere = awLike)  then  begin
      Result  :=  (AnsiPos(AnsiLowerCase(s2), AnsiLowerCase(s1)) > 0);
    end else
    if  (a.ActionWhere = awSame)  then  begin
      Result  :=  (AnsiLowerCase(Trim(s2)) = AnsiLowerCase(Trim(s1)));
    end else
    if  (a.ActionWhere = awLengthLT)  then  begin
      Result  :=  (Length(s1) < StrToInt(s2));
    end else
    if  (a.ActionWhere = awLengthGT)  then  begin
      Result  :=  (Length(s1) > StrToInt(s2));
    end else begin
      Result  :=  False;
    end;
  end;
var
  i : integer;
  x : integer;
  b : boolean;
  f : integer;
begin
  Result  :=  False;
  f       :=  CheckBegin;
  for i :=  f to  Count - 1 do  begin
    for x :=  0 to  Items[i].TargetBoardList.Count - 1  do  begin
      if  ((Items[i].TargetBoardList[x].SiteID  = SiteID) and
           (Items[i].TargetBoardList[x].BoardID = BoardID)) or
           (Items[i].TargetBoardList[x].IsAll) then
      begin
        b       :=  False;
        if  (not b) and (atResHandle in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], Res.Name);
        end;
        if  (not b) and (atResMail in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], Res.Mail);
        end;
        if  (not b) and (atResID in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], Res.ID);
        end;
        if  (not b) and (atResBody in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], Res.Body);
        end;
        if  (b) then  begin
          Action      :=  Items[i];
          CheckBegin  :=  i + 1;
          Result      :=  True;
          Exit;
        end;
      end;
    end;
  end;
end;

function TZonuActionList.CheckThread(ThreadItem: TThreadItem; SiteID,
  BoardID: string; var Action: TZonuAction): boolean;

  function  _IsNG(a: TZonuAction; const s: string): boolean;
  var
    s1: string;
    s2: string;
  begin
    s1  :=  DecodeHtmlEsc(s);
    s2  :=  StringReplace(a.Param1, '#13#10', #13#10, [rfReplaceAll]);
    if  (a.ActionWhere = awLike)  then  begin
      Result  :=  (AnsiPos(AnsiLowerCase(s2), AnsiLowerCase(s1)) > 0);
    end else
    if  (a.ActionWhere = awSame)  then  begin
      Result  :=  (AnsiLowerCase(Trim(s2)) = AnsiLowerCase(Trim(s1)));
    end else
    if  (a.ActionWhere = awLengthLT)  then  begin
      Result  :=  (Length(s1) < StrToInt(s2));
    end else
    if  (a.ActionWhere = awLengthGT)  then  begin
      Result  :=  (Length(s1) > StrToInt(s2));
    end else
    if  (a.ActionWhere = awCountGT)  then  begin
      Result  :=  (StrToInt(s1) > StrToInt(s2));
    end else
    if  (a.ActionWhere = awDateGT)  then  begin
      if  (s1 <> '')  then  begin
        Result  :=  ((Now - StrToDateTime(s1)) > StrToInt(s2));
      end else begin
        Result  :=  False;
      end;
    {
    end else
    if  (a.ActionWhere = awNonGetCountGT)  then  begin
      Result  :=  (StrToInt(s1) > StrToInt(s2));
    }
    end else begin
      Result  :=  False;
    end;
  end;
var
  i : integer;
  x : integer;
  b : boolean;
begin
  Result  :=  False;
  for i :=  0 to  Count - 1 do  begin
    for x :=  0 to  Items[i].TargetBoardList.Count - 1  do  begin
      if  ((Items[i].TargetBoardList[x].SiteID  = SiteID) and
           (Items[i].TargetBoardList[x].BoardID = BoardID)) or
           (Items[i].TargetBoardList[x].IsAll) then
      begin
        if  (Items[i].NewThreadOnly) then begin
          if  (not ThreadItem.IsNew)  then  begin
            Break;
          end;
        end;
        b   :=  False;
        if  (not b) and (atThreadTitle in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], ThreadItem.Title);
        end;
        if  (not b) and (atThreadResCount in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], IntToStr(ThreadItem.ReadCount));
        end;
        if  (not b) and (atThreadLoadDate in Items[i].FTarget) then  begin
          b :=  _IsNG(Items[i], ThreadItem.LastLoadDate);
        end;
        if  (not b) then  begin
          if  (Items[i].ActionWhere = awNonGetCountGT)  then  begin
            if  (ThreadItem.DownloadCount > 0)  then  begin
              b :=  ((ThreadItem.AgeCount - ThreadItem.DownloadCount) > StrToIntDef(Items[i].Param1 ,0)); 
            end;
          end;
        end;
        if  (b) then  begin
          Action  :=  Items[i];
          Result  :=  True;
          Exit;
        end;
      end;
    end;
  end;
end;

end.
