{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 Shortcut Bar Component Unit                                       }
{                                                                             }
{ 2001-02 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuShortcutBar;

interface

uses
  SysUtils, Classes, ExtCtrls, Buttons, ComCtrls, Controls, Graphics, Forms,
  Menus;

type
  TZonuShortcutBar    = class;
  TZonuShortcutPanel  = class;
  TZonuToolBar        = class;
  TZonuShortcutToolButton     = class;

  TObjectType = (otSeparator, otBoard, otThread, otApplication, otWebPage, otCategory);

  TZonuShortcutBar = class(TPanel)
  private
    FList: TList;
    FActiveTabIndex: integer;
    FActiveTab: TZonuShortcutPanel;
    FShortcutClickEvt: TNotifyEvent;
    FShortcutCreatedEvt: TNotifyEvent;
    procedure SetActiveTabIndex(const Value: integer);
    procedure SetActiveTab(const Value: TZonuShortcutPanel);
    procedure ShortcutClick(Sender: TObject);
    function GetTabs(Index: integer): TZonuShortcutPanel;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Add(Activate : boolean = True): TZonuShortcutPanel;
    procedure LoadFromFile(const FileName: string);
    procedure SaveToFile(const FileName: string);
    procedure InitTab;
    property  ActiveTabIndex: integer read  FActiveTabIndex write SetActiveTabIndex;
    property  ActiveTab: TZonuShortcutPanel read  FActiveTab write SetActiveTab;
    property  Tabs[Index: integer]: TZonuShortcutPanel  read  GetTabs;
  published
    property  OnShortcutClick: TNotifyEvent  read  FShortcutClickEvt write FShortcutClickEvt;
    property  OnShortcutCreated: TNotifyEvent  read  FShortcutCreatedEvt write FShortcutCreatedEvt;
  end;

  TZonuShortcutPanel = class(TPanel)
  private
    FTitleButton: TSpeedButton;
    FBackButton:  TSpeedButton;
    FForwardButton:  TSpeedButton;
    FToolBar: TZonuToolBar;
    FToolPanel: TPanel;
    FOldWidth: integer;
    FShortcutBar: TZonuShortcutBar;
    FTitle: string;
    FBitmap: string;
    FForwardTimer: TTimer;
    FBackTimer: TTimer;
    FLeftLine: TBevel;
    procedure SetBitmap(const Value: string);
    procedure SetTitle(const Value: string);
  protected
    procedure TitleButtonClick(Sender: TObject);
    procedure ForwardMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BackMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ForwardMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BackMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ForwardScroll(Sender: TObject);
    procedure BackScroll(Sender: TObject);
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Minimize;
    procedure Restore;
    procedure ShowLeftLine;
    property  ShortcutBar: TZonuShortcutBar read  FShortcutBar;
    property  ToolBar: TZonuToolBar read  FToolBar  write FToolBar;
    property  Title: string read  FTitle  write SetTitle;
    property  Bitmap: string  read  FBitmap write SetBitmap;
  end;

  TZonuToolBar = class(TPanel)
  private
    FList: TList;
    FInnerPanel: TPanel;
    function  GetControlsLength: integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function  AddButton: TZonuShortcutToolButton;
    procedure AddSeparator;
    procedure ScrollForward;
    procedure ScrollBack;
    function  ButtonCount:  integer;
    function  GetButton(Index: integer): TZonuShortcutToolButton;
    function  CanScrollForward: boolean;
    function  CanScrollBack: boolean;
  end;

  TZonuShortcutToolButton = class(TSpeedButton)
  private
    FObjectType: TObjectType;
    FTitle: string;
    FParam1: string;
    FParam3: string;
    FParam2: string;
    procedure SetTitle(const Value: string);
  protected
    procedure SetCaption(Value: string);
  public
    property  Caption: string   write SetCaption;
    property  Title: string read  FTitle  write SetTitle;
    property  Param1: string  read  FParam1 write FParam1;
    property  Param2: string  read  FParam2 write FParam2;
    property  Param3: string  read  FParam3 write FParam3;
    property  ObjectType: TObjectType read  FObjectType write FObjectType;
  end;
  {
    ************* ********* ********* *********
    ObjectType    Param1    Param2    Param3
    ************* ********* ********* *********
    Board         BoardID   SiteID    -
    Thread        DatNo     BoardID   SiteID
    Application   Path      -         -
    WebPage       URL       -         -
    Category      Category  SiteID    -
    ************* ********* ********* *********
  }


procedure Register;

implementation

{$RESOURCE ZonuShortcutBar.res}

procedure Register;
begin
  RegisterComponents('Samples', [TZonuShortcutBar]);
end;

{
uses
  MonaUtils, Zonutils;
}

const
  PANEL_WIDTH   = 465;
  PANEL_HEIGHT  = 27; //29;

function ChooseString(const Text, Separator: string; Index: integer): string;
var
  S : string;
  i, p : integer;
begin
  S :=  Text;
  for i :=  0 to  Index - 1 do  begin
    if  (AnsiPos(Separator, S) = 0) then  S :=  ''
    else  S :=  Copy(S, AnsiPos(Separator, S) + Length(Separator), Length(S));
  end;
  p :=  AnsiPos(Separator, S);
  if  (p > 0) then  Result  :=  Copy(S, 1, p - 1) else Result :=  S;
end;

////////////////////////////////////////////////////////////////////////////////
{ TZonuShortcutBar }
////////////////////////////////////////////////////////////////////////////////

constructor TZonuShortcutBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FList :=  TList.Create;
  FActiveTabIndex :=  -1;
  Height  :=  PANEL_HEIGHT;
  Caption :=  '';
  BevelInner  :=  bvNone;
  BevelOuter  :=  bvNone;
end;

destructor TZonuShortcutBar.Destroy;
var
  i : integer;
  sp: TZonuShortcutPanel;
begin
  for i :=  FList.Count - 1 downto  0 do  begin
    sp  :=  TZonuShortcutPanel(FList[i]);
    sp.Free;
  end;
  FList.Clear;
  FList.Free;
  inherited Destroy;
end;

function TZonuShortcutBar.Add(Activate : boolean = True): TZonuShortcutPanel;
var
  sp  : TZonuShortcutPanel;
  n : integer;
begin
  sp  :=  TZonuShortcutPanel.Create(Self);
  sp.Parent :=  Self;
  n   :=  FList.Add(sp);
  if  (Activate)  then  ActiveTabIndex :=  n;
  Result  :=  sp;
  if  (FList.Count > 1) then  begin
    sp.ShowLeftLine;
  end;

  Height  :=  sp.Height;
end;

procedure TZonuShortcutBar.SetActiveTabIndex(const Value: integer);
var
  i : integer;
begin
  if  (FActiveTabIndex <> Value)  then  begin
    if  (Value > 0) then  begin
      for i :=  0 to  Value - 1 do  begin
        TZonuShortcutPanel(FList[i]).Minimize;
        TZonuShortcutPanel(FList[i]).Align  :=  alLeft;
      end;
    end;
    if  (Value <  FList.Count - 1)  then  begin
      for i :=  FList.Count - 1 downto  Value + 1 do  begin
        TZonuShortcutPanel(FList[i]).Minimize;
        TZonuShortcutPanel(FList[i]).Align  :=  alRight;
      end;
    end;
    TZonuShortcutPanel(FList[Value]).Restore;
    TZonuShortcutPanel(FList[Value]).Align  :=  alClient;
    FActiveTabIndex := Value;
  end;
end;

procedure TZonuShortcutBar.SetActiveTab(const Value: TZonuShortcutPanel);
begin
  FActiveTab := Value;
  ActiveTabIndex  :=  FList.IndexOf(Value);
end;

function TZonuShortcutBar.GetTabs(Index: integer): TZonuShortcutPanel;
begin
  Result  :=  TZonuShortcutPanel(FList[Index]);
end;

procedure TZonuShortcutBar.LoadFromFile(const FileName: string);
var
  sl  : TStringList;
  sp  : TZonuShortcutPanel;
  tb  : TZonuShortcutToolButton;
  i   : integer;
  s, img : string;
begin
  FActiveTabIndex  :=  -1;
  sl  :=  TStringList.Create;
  if  (FileExists(FileName))  then  begin
    sl.LoadFromFile(FileName);
  end;
  for i :=  FList.Count - 1 downto  0 do  begin
    TZonuShortcutPanel(FList[i]).Free;
    FList.Delete(i);
  end;
  sp  :=  nil;
  for i :=  0 to  sl.Count - 1  do  begin
    if  (Length(sl[i]) > 0) then  begin
      if  (sl[i][1] <>  #9) then  begin
        //
        //  Vy[W쐬
        //
        sp        :=  Add(False);
        sp.Title  :=  ChooseString(sl[i], #9, 0);
        sp.Color  :=  StringToColor(ChooseString(sl[i], #9, 1));
        sp.ToolBar.Color      :=  sp.Color;
        sp.FToolPanel.Color   :=  sp.Color;
        SetCurrentDir(ExtractFilePath(Application.ExeName));
        img :=  ExpandFileName(ChooseString(sl[i], #9, 2));
        if  (FileExists(img)) then  begin
          sp.Bitmap :=  img;
        end;
      end else begin
        //
        //  {^쐬
        //
        s  :=  ChooseString(sl[i], #9, 1);
        if  (s = '-') or (s = '0') then  begin
          sp.ToolBar.AddSeparator;
        end else begin
          tb  :=  sp.ToolBar.AddButton;
          tb.ObjectType   :=  TObjectType(StrToInt(ChooseString(sl[i], #9, 1)));
          tb.Caption      :=  ChooseString(sl[i], #9, 2);
          tb.Title        :=  ChooseString(sl[i], #9, 2);
          tb.Param1       :=  ChooseString(sl[i], #9, 3);
          tb.Param2       :=  ChooseString(sl[i], #9, 4);
          tb.Param3       :=  ChooseString(sl[i], #9, 5);
          tb.OnClick      :=  ShortcutClick;
          if  (Assigned(FShortcutCreatedEvt)) then  begin
            FShortcutCreatedEvt(tb);
          end;
        end;
      end;
    end;
  end;
  if  (sl.Count > 0)  then  ActiveTabIndex  :=  0;
  sl.Free;
end;

procedure TZonuShortcutBar.ShortcutClick(Sender: TObject);
begin
  if  (Assigned(FShortcutClickEvt)) then  begin
    FShortcutClickEvt(Sender);
  end;
end;

procedure TZonuShortcutBar.SaveToFile(const FileName: string);
var
  sl  : TStringList;
  sp  : TZonuShortcutPanel;
  tb  : TZonuShortcutToolButton;
  i, x : integer;
begin
  sl  :=  TStringList.Create;
  for i :=  0 to  FList.Count - 1 do  begin
    sp  :=  TZonuShortcutPanel(FList[i]);
    sl.Add(sp.Title + #9 + ColorToString(sp.Color) + #9 + ExtractRelativePath(ExtractFileName(Application.ExeName), sp.Bitmap));
    for x :=  0 to  sp.ToolBar.ButtonCount - 1  do  begin
      tb  :=  sp.ToolBar.GetButton(x);
      sl.Add(Format(#9 + '%d' + #9 + '%s' + #9 + '%s' + #9 + '%s' + #9 + '%s',
             [Integer(tb.ObjectType), tb.Title, tb.Param1, tb.Param2, tb.Param3]));
    end;
  end;
  sl.SaveToFile(FileName);
  sl.Free;
end;

procedure TZonuShortcutBar.InitTab;
var
  a : integer;
begin
  a :=  ActiveTabIndex;
  FActiveTabIndex :=  -1;
  ActiveTabIndex  :=  a;
end;

////////////////////////////////////////////////////////////////////////////////
{ TZonuShortcutPanel }
////////////////////////////////////////////////////////////////////////////////

procedure TZonuShortcutPanel.BackMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FBackTimer.Enabled :=  True;
end;

procedure TZonuShortcutPanel.BackMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FBackTimer.Enabled :=  False;
end;

procedure TZonuShortcutPanel.BackScroll(Sender: TObject);
begin
  ToolBar.ScrollBack;
  Resize;
end;

constructor TZonuShortcutPanel.Create(AOwner: TComponent);
begin
  inherited;
  Caption     :=  '';
  BevelInner  :=  bvNone;
  BevelOuter  :=  bvNone;
  Width       :=  PANEL_WIDTH;
  Height      :=  PANEL_HEIGHT - 4;
  Color       :=  clBtnFace;

  FLeftLine   :=  TBevel.Create(Self);
  FLeftLine.Parent  :=  Self;
  FLeftLine.Align   :=  alLeft;
  FLeftLine.Shape   :=  bsLeftLine;
  FLeftLine.Hide;

  FTitleButton        :=  TSpeedButton.Create(Self);
  FTitleButton.Parent :=  Self;
  FTitleButton.Top  :=  0;  //2;
  FTitleButton.Left :=  4;
  FTitleButton.Flat     :=  True;
  FTitleButton.OnClick  :=  TitleButtonClick;
  FTitleButton.Caption  :=  '';

  FToolPanel    :=  TPanel.Create(Self);
  FToolPanel.Parent :=  Self;
  FToolPanel.SetBounds(28, 0,{ 2,} PANEL_WIDTH - 28, 22);
  FToolPanel.Caption  :=  '';
  FToolPanel.BevelInner  :=  bvNone;
  FToolPanel.BevelOuter  :=  bvNone;
  FToolPanel.Anchors  :=  [akTop, akLeft, akRight];

  FForwardButton :=  TSpeedButton.Create(FToolPanel);
  FForwardButton.Parent   :=  FToolPanel;
  FForwardButton.Top      :=  0;
  FForwardButton.Left     :=  FToolPanel.Width - 13;
  FForwardButton.Width    :=  13;
  FForwardButton.Caption  :=  '';
  FForwardButton.Flat     :=  True;
  FForwardButton.Anchors  :=  [akTop, akRight];
  FForwardButton.Glyph.LoadFromResourceName(hInstance, 'BMPFORWARD');
  FForwardButton.OnMouseDown :=  ForwardMouseDown;
  FForwardButton.OnMouseUp   :=  ForwardMouseUp;
  FForwardButton.Enabled     :=  False;

  FBackButton   :=  TSpeedButton.Create(FToolPanel);
  FBackButton.Parent    :=  FToolPanel;
  FBackButton.Top       :=  0;
  FBackButton.Left      :=  1;
  FBackButton.Width     :=  13;
  FBackButton.Caption   :=  '';
  FBackButton.Flat      :=  True;
  FBackButton.Glyph.LoadFromResourceName(hInstance, 'BMPBACK');
  FBackButton.OnMouseDown :=  BackMouseDown;
  FBackButton.OnMouseUp   :=  BackMouseUp;
  FBackButton.Enabled     :=  False;

  FToolBar    :=  TZonuToolBar.Create(FToolPanel);
  FToolBar.Parent        :=  FToolPanel;
  FToolBar.SetBounds(14, 0,  FToolPanel.Width - 30, 22);
  FToolBar.BorderStyle   :=  bsNone;
  FToolBar.Anchors       :=  [akTop, akLeft, akRight];

  FForwardTimer :=  TTimer.Create(Self);
  FForwardTimer.Enabled   :=  False;
  FForwardTimer.Interval  :=  50;
  FForwardTimer.OnTimer   :=  ForwardScroll;

  FBackTimer :=  TTimer.Create(Self);
  FBackTimer.Enabled    :=  False;
  FBackTimer.Interval   :=  50;
  FBackTimer.OnTimer    :=  BackScroll;

  FShortcutBar  :=  TZonuShortcutBar(AOwner);
end;

destructor TZonuShortcutPanel.Destroy;
begin
  inherited Destroy;
end;

procedure TZonuShortcutPanel.ForwardMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FForwardTimer.Enabled :=  True;
end;

procedure TZonuShortcutPanel.ForwardMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FForwardTimer.Enabled :=  False;
end;

procedure TZonuShortcutPanel.ForwardScroll(Sender: TObject);
begin
  ToolBar.ScrollForward;
  Resize;
end;

procedure TZonuShortcutPanel.Minimize;
begin
  FToolPanel.Hide;
  FOldWidth :=  Width;
  Align :=  alNone;
  Width :=  28;
end;

procedure TZonuShortcutPanel.Resize;
var
  b: boolean;
begin
  inherited;
  FBackButton.Enabled     :=  (FToolBar.CanScrollBack);
  FForwardButton.Enabled  :=  (FToolBar.CanScrollForward);
  b :=  FBackButton.Visible;
  FBackButton.Visible     :=  FBackButton.Enabled;
  if  (b <> FBackButton.Visible)  then  begin
    if  (FBackButton.Visible) then  begin
      FToolBar.Left   :=  14;
      FToolBar.Width  :=  FToolBar.Width -  10;
    end else begin
      FToolBar.Left   :=  4;
      FToolBar.Width  :=  FToolBar.Width +  10;
    end;
  end;
end;

procedure TZonuShortcutPanel.Restore;
begin
  FToolPanel.Show;
  if  (FOldWidth > 0) then  Width :=  FOldWidth;
end;

procedure TZonuShortcutPanel.SetBitmap(const Value: string);
begin
  FBitmap := Value;
  FTitleButton.Glyph.LoadFromFile(Value);
end;

procedure TZonuShortcutPanel.SetTitle(const Value: string);
begin
  FTitle := Value;
  FTitleButton.Hint     :=  Value;
  FTitleButton.ShowHint :=  True;
end;

procedure TZonuShortcutPanel.ShowLeftLine;
begin
  FLeftLine.Show;
end;

procedure TZonuShortcutPanel.TitleButtonClick(Sender: TObject);
begin
  ShortcutBar.ActiveTab :=  Self;
end;

{ TZonuToolBar }

constructor TZonuToolBar.Create(AOwner: TComponent);
begin
  inherited;
  FList :=  TList.Create;
  BevelInner  :=  bvNone;
  BevelOuter  :=  bvNone;
  Caption     :=  '';
  ParentColor :=  True;

  FInnerPanel :=  TPanel.Create(Self);
  FInnerPanel.Parent  :=  Self;
  FInnerPanel.SetBounds(0,0,Width, Height);
  FInnerPanel.BevelInner  :=  bvNone;
  FInnerPanel.BevelOuter  :=  bvNone;
  FInnerPanel.Caption     :=  '';
  FInnerPanel.ParentColor :=  True;
end;

destructor TZonuToolBar.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

function TZonuToolBar.AddButton: TZonuShortcutToolButton;
var
  tb  : TZonuShortcutToolButton;
  //m   : TMenuItem;
begin
  tb    :=  TZonuShortcutToolButton.Create(Self);
  tb.Parent :=  FInnerPanel;
  tb.Top    :=  0;
  tb.Width  :=  50;
  tb.Left   :=  GetControlsLength;
  tb.Flat   :=  True;

  Result    :=  tb;
  FInnerPanel.Width :=  tb.Left + tb.Width + 50;

  FList.Add(tb);
end;

procedure TZonuToolBar.AddSeparator;
var
  bv  : TBevel;
begin
  bv    :=  TBevel.Create(Self);
  bv.Parent :=  FInnerPanel;
  bv.Top    :=  0;
  bv.Left   :=  GetControlsLength + 1;
  bv.Width  :=  3;
  bv.Shape  :=  bsLeftLine;
  FInnerPanel.Width :=  bv.Left + bv.Width;

  FList.Add(bv);
end;

function TZonuToolBar.GetControlsLength: integer;
var
  r : integer;
begin
  r :=  0;
  if  (FList.Count > 0) then  begin
    r :=  TGraphicControl(FList[FList.Count - 1]).Left +
          TGraphicControl(FList[FList.Count - 1]).Width; 
  end;
  Result  :=  r;
end;

procedure TZonuToolBar.ScrollBack;
begin
  if  (FInnerPanel.Left < 0)  then  begin
    FInnerPanel.Left  :=  FInnerPanel.Left  + 20;
  end else begin
    FInnerPanel.Left  :=  0;
  end;
end;

procedure TZonuToolBar.ScrollForward;
begin
  if  (-FInnerPanel.Left > FInnerPanel.Width - Width)  then  begin

  end else begin
    FInnerPanel.Left  :=  FInnerPanel.Left  - 20;
  end;
end;

function TZonuToolBar.ButtonCount: integer;
begin
  Result  :=  FList.Count;
end;

function TZonuToolBar.GetButton(Index: integer): TZonuShortcutToolButton;
begin
  Result  :=  TZonuShortcutToolButton(FList[Index]);
end;

function TZonuToolBar.CanScrollBack: boolean;
begin
  Result  :=  (FInnerPanel.Left < 0);
end;

function TZonuToolBar.CanScrollForward: boolean;
begin
  Result  :=  (FInnerPanel.Width + FInnerPanel.Left > Width);
end;

{ TZonuShortcutToolButton }

procedure TZonuShortcutToolButton.SetCaption(Value: string);
begin
  inherited;
  TSpeedButton(Self).Caption  :=  Value;
  Width :=  Canvas.TextWidth(Value) + 8;
end;

procedure TZonuShortcutToolButton.SetTitle(const Value: string);
begin
  FTitle  :=  Value;
  Caption :=  Value;
end;

end.
