{ *************************************************************************** }
{                                                                             }
{ Hotzonu AhhanBBS Script Unit                                                }
{                                                                             }
{ 2001-02 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuAhhan;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst;

type
  PParamData  = ^TParamData;
  TParamData  = record
    Subject : string;
    InDat : boolean;
    TempData: string;
  end;

  TAhhanScript = class(TBBS)
  private
    { private }
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
  public
    { public }
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, Async, InfSave: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, Async, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
  end;

implementation

{ TAhhanScript }

//
//  Xbhꗗ̎擾@
//
function TAhhanScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

//
//  Xbh̎擾@
//
function TAhhanScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmCGI;
end;

//
//  http://ւ̃Nǉ
//
function TAhhanScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function TAhhanScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function TAhhanScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure TAhhanScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, Async, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', Async);
end;

//
//  Xbhꗗ擾
//
procedure TAhhanScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
begin
  inherited;
  if  (AnsiPos('<font title= ', AnsiLowerCase(Data)) > 0)  then
  begin
    dl  :=  TThreadListDownloader(Downloader);
    ParseSubject(Data, AFileName, ATitle, ACount);
    ThreadItem                    :=  TThreadItem.Create;
    ThreadItem.BBS                :=  Self;
    ThreadItem.Site               :=  dl.Site;
    ThreadItem.BoardID            :=  dl.Board.BoardID;
    ThreadItem.No                 :=  0;
    ThreadItem.DatFileName        :=  AFileName;
    ThreadItem.AgeCount           :=  ACount;
    ThreadItem.Title              :=  ATitle;
    ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
    ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName + '.dat';
    dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
  end;
end;

//
//  Xbhꗗ擾I
//
procedure TAhhanScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure TAhhanScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  Xbhꗗ̉
//
procedure TAhhanScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  ParseAhhanSubject(subject, FileName, Title, Count);
end;

//
//  X̉
//
procedure TAhhanScript.ParseDat(const ALine: string; const index: integer;
  var ARes: TRes; out ASubject: string);
begin
  inherited;
  ParseAhhanDat(ALine, Index, ARes, ASubject);
end;

//
//  \URL
//
function TAhhanScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  Result  :=  GetReadCgiUrl(ThreadItem);
end;

//
//  Xbh̃_E[h
//
procedure TAhhanScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, Async, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  ParamData.Subject         :=  '';
  ParamData.InDat           :=  False;
  ParamData.TempData        :=  '';

  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.Url             :=  Downloader.BBS.GetReadCgiUrl(Downloader.ThreadItem);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  FZonuHttp.GetCgiDiff(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), Async);
  Downloader.HttpObj        :=  FZonuHttp;
end;

//
//  X擾
//
procedure TAhhanScript.ZonuDataRes(Sender: TObject; Downloader: TDownloader;
  Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean);
var
  dl  : TResDownloader;
  ParamData:  PParamData;
  ASubject :  string;
  Res : TRes;
  s, sx : string;
  m, n : integer;
begin
  dl  :=  TResDownloader(Downloader);
  ParamData :=  PParamData(Downloader.ParamData);
  ParamData.TempData  :=  ParamData.TempData  + Data;
  if  (AnsiPos('<dt>', Data) = 1) then  begin
    ParamData.InDat   :=  True;
    ParamData.TempData:=  Data; //Jn
  end else
  if  ((AnsiPos('<p>', Trim(Data)) = 1) and (ParamData.InDat))  or
      (AnsiPos('<dt>', Data) = 1) then
  begin
    Res :=  TRes.Create;
    ParseDat(ParamData.TempData, 0, Res, ASubject);
    Res.BBS         :=  dl.BBS;
    Res.Site        :=  dl.Site;
    Res.ThreadTitle :=  ParamData.Subject;
    dl.FireDataEvt(Res, DownloadSize, TotalSize);
    if  (dl.ThreadItem.Title = '')  then  begin
      if  (ParamData.Subject <> '')  then  begin
        dl.ThreadItem.Title :=  ParamData.Subject;
      end;
    end;
    ParamData.TempData  :=  '';
    ParamData.InDat     :=  False;
  end;
  if  (AnsiPos('<title>', Data) = 1) then  begin
    s   :=  Copy(Data, 8, Length(Data));
    sx  :=  s;
    n   :=  0;
    //݂Ȃ̂ajinQPQCH212chBBS
    while true  do  begin
      m :=  AnsiPos('', sx);
      if  (m < 1) then  Break;
      n   :=  n + m + 1;
      sx  :=  Copy(s, n + 1, Length(sx));
    end;
    if  (n > 0) then  begin
      ParamData.Subject :=  Copy(s, 1, n - 2);
    end else begin
      ParamData.Subject :=  s;
    end;
  end;
end;

//
//  X擾I
//
procedure TAhhanScript.ZonuEndRes(Sender: TObject; Downloader: TDownloader;
  Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure TAhhanScript.ZonuResError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  
//
procedure TAhhanScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  FZonuHttp.Send;
end;

//
//  VX
//
procedure TAhhanScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  FZonuHttp.Send;
end;

function TAhhanScript.GetRelativeLink: boolean;
begin
  Result  :=  True;
end;

end.
